# -*- coding:utf-8 -*-

from .XLSReader import XLSLoader


SUCCESFUL_IMPORTED = u'Файл успешно импортирован'


class ProxyLoader(object):
    """Загрузка файла по словарю прокси-загрузчиков."""

    # Класс отчета об ошибках
    report_cls = None

    # Сообщение о результате
    message = None

    # Адрес отчета об ошибках
    report_url = None

    def __init__(self, loaders, _file, file_name, context=None,
                 separate_logs=False):
        u"""Загрузка файла по словарю прокси-загрузчиков.

        :param dict loaders: словарь постраничных загрузчиков файла
        :param InMemoryUploadedFile _file: файл загрузки в оперативной памяти
        :param unicode file_name: название файла
        :param dict context: контекст
        :param bool separate_logs: флаг разделения логов импорта
        """

        config = {}
        for key, loader in loaders.iteritems():
            config[key] = loader.make_config() if loader else {}

        self.xls_loader = XLSLoader(_file, config)
        self.loaders = loaders
        self.context = context or {}
        self.context['XLS_POS'] = self.xls_loader.XLS_POS
        self.separate_logs = separate_logs

    def make_log(self, log):
        """
        Сбор лога для окна результата
        :param dict log: лог
        :rtype str
        """
        return u'\n'.join(
            self.xls_loader.log + self.xls_loader.prepare_row_errors(log))

    def load(self):
        """Загрузка"""
        if not self.xls_loader.load():
            self.message = self.make_log(self.xls_loader.rows_log)
            return True

        # Логи разделены с целью дать возможность загрузчику принять решение о
        # дальнейшей загрузке если были ошибки парсинга ячеек лог разбора ячеек
        parse_log = self.xls_loader.rows_log

        # сквозной лог загрузки данных проксями будет дополнен parse_log'ом в
        # методе load_rows у загрузчика листа
        import_log = {}

        # опционально - сквозной лог для предупреждений
        warning_log = {} if self.separate_logs else None

        for sheet, data in self.xls_loader.data.iteritems():
            header = self.xls_loader.headers.get(sheet, [])
            # Класс загрузчика листа
            loader_cls = self.loaders.get(sheet)

            if not loader_cls:
                # Если не найден загрузчик для листа
                import_log[data[0][self.xls_loader.XLS_POS]] = (
                    u'Некорректное имя листа "%s"' % sheet
                )
                continue

            # выполняет загрузку листа (с разбором шапки и логированием)
            loader_cls.load_rows(
                header, data, parse_log, import_log, self.context, warning_log)

        log = self.make_log(import_log)
        result = not bool(log)

        if self.separate_logs:
            # если задан режим разделения логов
            if log:
                # всё же были какие-то ошибки
                final_log = [u" ОШИБКИ:\n ", log]
            else:
                final_log = [SUCCESFUL_IMPORTED, ]

            if warning_log:
                final_log.extend([
                    u" ПРЕДУПРЕЖДЕНИЯ:\n ", self.make_log(warning_log)
                ])

            self.message = u'\n'.join(final_log)
        else:
            self.message = log or SUCCESFUL_IMPORTED

        # Если произошла ошибка, составим отчет
        if not result and self.report_cls:
            report = self.report_cls(import_log)
            self.report_url = report.make()

        return result
